# -*- coding: utf-8 -*-
"""API shortcuts."""
from django_confit.loaders import load_module  # NoQA
from django_confit.loaders import load_file  # NoQA
from django_confit.loaders import load_mapping  # NoQA
from django_confit.schemas import DjangoConfigurationSchema  # NoQA
from django_confit.schemas import validate_settings  # NoQA
